function gap=vgap(B1,A1,B2,A2,tol)
% vgap computes vinnicombe's gap between two descrete time SISO transfer function
%
%		gap=vgap(B1,A1,B2,A2,tol)
%
%		B1 and A1 are respectively the numerator and denominator of the first 
%		transfer function
%		B2 and A2 are respectively the numerator and denominator of the second 
%		transfer function
%
%		tol is the tolerance value (default=0.001)
%
% June 30,1999
% Copyright by Adaptech, 1997-1999

if nargin<4, error('usage: gap=vgap(B1,A1,B2,A2,tol)');end
if nargin<5, tol=0.001;end

[r,c]=size(B1);if r>c, B1=B1';end
[r,c]=size(B2);if r>c, B2=B2';end
[r,c]=size(A1);if r>c, A1=A1';end
[r,c]=size(A2);if r>c, A2=A2';end


gap=1;

if length(A1)>length(B1), B1=[B1 zeros(1,length(A1)-length(B1))];
else A1=[A1 zeros(1,length(B1)-length(A1))];end

if length(A2)>length(B2), B2=[B2 zeros(1,length(A2)-length(B2))];
else A2=[A2 zeros(1,length(B2)-length(A2))];end

% Compute the conjucate of P2=B2/A2
n=length(A2);
A2bar=[];B2bar=[];
for i=1:n, A2bar=[A2(i) A2bar];B2bar=[B2(i) B2bar];end


% Compute 1+P2*P1
Den=conv(A2bar,A1);
Num=Den+conv(B2bar,B1);

w=logspace(-4,pi,round(1/tol));
z=exp(-j*w);

m=abs(polyval(Num,z)./polyval(Den,z));
if max(m)*min(m)<=0, return,end

% compute the winding number
etaP1=0;etaP2=0;eta0P2=0;
etaNum=0;etaDen=0;

r1=abs(roots(A1));
if max(r1)>1,
	for i=1:length(r1), 
   	if r1(i)>1, etaP1=etaP1+1;end;
	end
end

r2=abs(roots(A2));
if max(r2)>=1,
	for i=1:length(r2), 
   	if r2(i)>1, etaP2=etaP2+1;end
   	if r2(i)==1, eta0P2=eta0P2+1;end
	end
end

r3=abs(roots(Num));
if max(r3)>1,
	for i=1:length(r3), 
   	if r3(i)>1, etaNum=etaNum+1;end;
	end
end
   
r4=abs(roots(Den));
if max(r4)>1,
	for i=1:length(r4), 
   	if r4(i)>1, etaDen=etaDen+1;end;
	end
end

wno=etaNum-etaDen+etaP1-etaP2-eta0P2;

if wno~=0, return,end


P1mP2=conv(B1,A2)-conv(B2,A1);
m1=abs(polyval(B1,z)./polyval(A1,z));
m2=abs(polyval(B2,z)./polyval(A2,z));
m3=abs(polyval(P1mP2,z)./polyval(conv(A1,A2),z));

gap=max(m3./sqrt((1+m1.*m1).*(1+m2.*m2)));

